SystemOrganization addCategory: #FixUnderscores!

!RGMethodDefinition methodsFor: '*FixUnderscores' stamp: ''!
fixLF
	"Replace underscores with :=. Answer true if fixed or no fix necessary, false if manual fix required"

	| src cm ts |
	"Check if we do need to do anything"
	src := self realClass sourceCodeAt: self selector.
	(src includes: $_) ifFalse: [^true].

	"Chicken out if there is a literal underscore"
	cm := self realClass compiledMethodAt: self selector.
	(cm hasLiteralSuchThat: [:lit | 
		lit = $_ or: [lit isString and: [lit includes: $_]]]) ifTrue: [^false].

	"Otherwise, replace underscores with :="
	src := src copyReplaceAll: Character lf asString with: Character space asString, Character cr asString.
	ts := self timeStamp.
	ts = '' ifTrue: [ts := nil].
	self actualClass 
		compile: src
		classified: ClassOrganizer default
		withStamp: ts 
		notifying: nil.

	^true
! !

!RGMethodDefinition methodsFor: '*FixUnderscores' stamp: ''!
fixUnderscores
	"Replace underscores with :=. Answer true if fixed or no fix necessary, false if manual fix required"

	| src cm ts |
	"Check if we do need to do anything"
	src := self realClass sourceCodeAt: self selector.
	(src includes: $_) ifFalse: [^true].

	"Chicken out if there is a literal underscore"
	cm := self realClass compiledMethodAt: self selector.
	(cm hasLiteralSuchThat: [:lit | 
		lit = $_ or: [lit isString and: [lit includes: $_]]]) ifTrue: [^false].

	"Otherwise, replace underscores with :="
	src := src copyReplaceAll: '_' with: ':='.
	ts := self timeStamp.
	ts = '' ifTrue: [ts := nil].
	self actualClass 
		compile: src
		classified: ClassOrganizer default
		withStamp: ts 
		notifying: nil.

	^true
! !

Object subclass: #FixUnderscores
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FixUnderscores'!

!FixUnderscores class methodsFor: 'fixing' stamp: 'bf 4/7/2006 14:47'!
arrowChar
	"FIXME: this should rather be (Character leftArrow)"
	^Character value: 16r8F! !

!FixUnderscores class methodsFor: 'tests' stamp: 'sd 3/16/2008 15:29'!
asSeconds
	"Answer the seconds since the Squeak epoch: 1 January 1901"

	^ 12 asSeconds! !

!FixUnderscores class methodsFor: 'fixing' stamp: 'bf 10/18/2005 19:05'!
fixFont: aFont
	| glyph underline |
	glyph := aFont characterFormAt: $_.
	"save arrow glyph to arrowChar codepoint"
	((glyph copy: (0@aFont ascent corner: glyph extent)) isAllWhite
		and: [(aFont characterFormAt: self arrowChar) isAllWhite])
			ifTrue: [aFont characterFormAt: self arrowChar put: glyph].
	"make underscore glyph"
	glyph fillWhite.
	underline := aFont ascent + 1.
	glyph fillBlack: (1@underline extent: glyph width-1@1).
	aFont characterFormAt: $_ put: glyph.! !

!FixUnderscores class methodsFor: 'fixing' stamp: 'bf 11/26/2004 21:52'!
fixFonts
	"self fixFonts"

	StrikeFont allInstances
		do: [:fnt | self fixFont: fnt]
		displayingProgress: 'Fixing Bitmap Fonts'.! !

!FixUnderscores class methodsFor: 'fixing' stamp: 'GuillermoPolito 8/9/2012 18:27'!
fixLFPackages: packageNames
	"FixUnderscores fixLFPackages: #('FixUnderscores')"

	| failed |
	failed := OrderedCollection new.

	packageNames
		do: [:pkgName | (PackageOrganizer default packageNamed: pkgName) methods
			do: [:mRef | mRef fixLFInvisible ifFalse: [failed add: mRef]] 
			displayingProgress: pkgName]
		displayingProgress: 'Fixing ...'.

	failed isEmpty ifFalse: [
		Smalltalk tools messageList 
			openMessageList: failed
			name: 'These methods with lf were not fixed'
			autoSelect: Character lf asString].! !

!FixUnderscores class methodsFor: 'fixing' stamp: 'bf 4/7/2006 15:11'!
fixPackage: aPackageName
	^self fixPackages: {aPackageName}

! !

!FixUnderscores class methodsFor: 'fixing' stamp: 'GuillermoPolito 8/9/2012 18:27'!
fixPackages: packageNames
	"FixUnderscores fixPackages: #('FixUnderscores' 'Bert')"

	| failed |
	failed := OrderedCollection new.

	packageNames
		do: [:pkgName | (PackageOrganizer default packageNamed:  pkgName) methods
			do: [:mRef | mRef fixUnderscores ifFalse: [failed add: mRef]] 
			displayingProgress: pkgName]
		displayingProgress: 'Fixing ...'.

	failed isEmpty ifFalse: [
		Smalltalk tools messageList 
			openMessageList: failed
			name: 'These methods with literal underscores were not fixed'
			autoSelect: '_'].! !

!FixUnderscores class methodsFor: 'initialization' stamp: 'IgorStasenko 4/15/2011 15:34'!
initialize
	"self initialize"

	self fixFonts.
"
Do not inform, while running non-interactively
	self inform: 'Fonts were _fixed_.\The arrow glyph is now Character value ' withCRs,
		self arrowChar asInteger hex, ' ($', self arrowChar asString, ')'.
"! !
FixUnderscores initialize!
